from inicheck.checkers import GenericCheck
from .health_check import check_missing_vars, check_missing_file, check_min_file_size
import netCDF4 as nc
import os
import glob
from collections import OrderedDict

class MissingVarCheck(GenericCheck):
    def __init__(self, **kwargs):
        super(MissingVarCheck, self).__init__(**kwargs)

    def cast(self):
        return self.value

    def is_valid(self):
        # path to file with pattern
        pattern = self.config.cfg['output_check']['pattern']
        file = self.config.cfg['output_check']['file']
        vars = self.config.cfg['output_check']['netcdf_vars']
        # find files with missing variables and the missing variables
        self.incomplete = check_missing_vars(pattern, file, vars)

        # if we have errors, construct the message
        if len(self.incomplete) > 0:
            self.message=('Variables missing from netcdfs.\n'
                          '{}\n'.format('\n'.join(['{} --- {}'.format(k,v) for k,v in self.incomplete.items()])))
            return False, self.message
        else:
            return True, self.message


class MissingFileCheck(GenericCheck):
    def __init__(self, **kwargs):
        super(MissingFileCheck, self).__init__(**kwargs)

    def cast(self):
        return self.value

    def is_valid(self):
        # path to file with pattern
        pattern = self.config.cfg['output_check']['pattern']
        file = self.config.cfg['output_check']['file']
        # find files with missing variables and the missing variables
        self.incomplete = check_missing_file(pattern, file)

        if len(self.incomplete) > 0:
            self.message=('Files missing from pattern.\n'
                          '{}\n'.format('\n'.join(['{}'.format(k) for k in self.incomplete])))
            return False, self.message
        else:
            return True, self.message


class MinSizeCheck(GenericCheck):
    def __init__(self, **kwargs):
        super(MinSizeCheck, self).__init__(**kwargs)

    def cast(self):
        return self.value

    def is_valid(self):
        # path to file with pattern
        pattern = self.config.cfg['output_check']['pattern']
        file = self.config.cfg['output_check']['file']
        min_size = self.config.cfg['output_check']['min_size']
        # find files with missing variables and the missing variables
        self.incomplete = check_min_file_size(pattern, file, min_size=min_size)

        if len(self.incomplete) > 0:
            message = 'Files smaller than {} kB.\n'.format(min_size)
            message = message + '{}\n'.format('\n'.join(['{}'.format(k) for k in self.incomplete]))
            self.message = message
            return False, self.message
        else:
            return True, self.message



from inicheck.config import MasterConfig, UserConfig
from inicheck.tools import get_user_config, check_config
from inicheck.output import print_config_report, generate_config
from inicheck.tools import cast_all_variables
import os


class HealthCheck():
    """
    Args:
        configFile (str):  path to configuration file.

    Returns:
        drhawkeye

    Attributes:
    """

    def __init__(self, config):

        if isinstance(config, str):
            if not os.path.isfile(config):
                raise Exception('Configuration file does not exist --> {}'
                                .format(config))

            configFile = config

            try:
                mcfg = MasterConfig(modules = 'drhawkeye')

                # Read in the original users config
                self.ucfg = get_user_config(configFile, mcfg=mcfg)
                self.configFile = configFile

            except UnicodeDecodeError as e:
                print(e)
                raise Exception(('The configuration file is not encoded in '
                                 'UTF-8, please change and retry'))

        elif isinstance(config, UserConfig):
            self.ucfg = config
            configFile = ''

        else:
            raise Exception('Config passed to drhawkeye is neither file name nor UserConfig instance')

        # Check the user config file for errors and report issues if any
        print("Checking config file for issues...")
        warnings, errors = check_config(self.ucfg)
        print_config_report(warnings, errors)

        self.config = self.ucfg.cfg

        # Exit AWSM if config file has errors
        if len(errors) > 0:
            print("Errors in the config file. "
                  "See configuration status report above.")
            sys.exit()



# -*- coding: utf-8 -*-
import os

__author__ = """Micah Sandusky"""
__email__ = 'micah.sandusky@ars.usda.gov'
__version__ = '0.2.4'

__core_config__ = os.path.abspath(os.path.dirname(__file__) + '/CoreConfig.ini')
__recipes__ = os.path.abspath(os.path.dirname(__file__) + '/recipes.ini')
__config_checkers__ = 'checkers'

from . import checkers
from . import framework
from . import health_check



"""
Create a series of checks that will analyze past AWSM runs and raise issues
including missing output files, files that are too small, etc
"""

from datetime import datetime
import sys
import os
import glob
from collections import OrderedDict
import netCDF4 as nc

def check_missing_file(pattern, file):
    """
    Find all directories matching the pattern that are missing the file

    Args:
        pattern:    Directory pattern with path to file location
        file:       File name

    Returns:
        missing:    List of directories missing the file name
    """
    # path to file with pattern
    pf = os.path.join(pattern, file)
    # find list of directories matching pattern
    d_dir = sorted(glob.glob(pattern), key=os.path.getmtime)
    d_dir.sort(key=lambda f: os.path.splitext(f))
    d_dir = list(set(d_dir))
    d_dir = [os.path.abspath(d) for d in d_dir]
    # find list of directories that have the file
    d_fp = sorted(glob.glob(pf), key=os.path.getmtime)
    d_fp.sort(key=lambda f: os.path.splitext(f))
    d_fp = [os.path.dirname(d) for d in d_fp]
    d_fp = list(set(d_fp))
    # find differences in the lists
    missing = [os.path.join(d, file) for d in d_dir if d not in d_fp]
    # return the directories missing the file

    return missing


def check_min_file_size(pattern, file, min_size=1000.0):
    """
    Check the size of all files matching pattern and filename and compare
    against a minimum file size in kB

    Args:
        pattern:    Directory pattern with path to file location
        file:       File name
        min_size:   Minimum file size in kB

    Returns:
        too_small:    List of snow files that are smaller than threshold
    """
    # path to file with pattern
    pf = os.path.join(pattern, file)
    d_fp = sorted(glob.glob(pf), key=os.path.getmtime)
    d_fp.sort(key=lambda f: os.path.splitext(f))
    #sz = [os.path.getsize(d) for d in d_fp]
    # find the files that don't meet the minimum size requirements
    too_small = [d for d in d_fp if os.path.getsize(d)/1000.0 < min_size]

    return too_small


def check_missing_vars(pattern, file, vars):
    """
    Check netcdf files for variables that should be present. Return
    ordered dictionary of files and their missing variables

    Args:
        pattern:    Directory pattern with path to file location
        file:       File name
        vars:       List of vars to check

    Returns:
        incomplete:     Ordered dictionary of files and their missing variables
    """

    # path to file with pattern
    pf = os.path.join(pattern, file)
    d_fp = sorted(glob.glob(pf), key=os.path.getmtime)
    d_fp.sort(key=lambda f: os.path.splitext(f))

    # empty dictionary
    incomplete = OrderedDict()

    for idd, d in enumerate(d_fp):
        # read in the netcdf and get variables
        ds = nc.Dataset(d, 'r')
        keys = ds.variables.keys()
        ds.close()

        # checks keys against list of vars
        mv = [v for v in vars if v not in keys]
        if len(mv) > 0:
            incomplete[d] = mv

    return incomplete


# if __name__ == '__main__':
#
#     ### Inputs ###
#     dir_pattern = '/data/blizzard/tuolumne/devel/wy2017/local_gradient/runs/run*'
#     file = 'snow.nc'
#     min_size = 1000.0
#     vars = ['thickness', 'snow_density', 'specific_mass', 'liquid_water',
#             'temp_surf', 'temp_lower', 'temp_snowcover', 'thickness_lower',
#             'water_saturation']
#
#     ### Done with inputs ###
#     missing = check_missing_file(dir_pattern, file)
#
#     too_small = check_min_file_size(dir_pattern, file, min_size=min_size)
#
#     incomplete = check_missing_vars(dir_pattern, file, vars)
#
#     print('\nDirectories missing {} file:'.format(file))
#     for li in missing:
#         print(li)
#
#     print('\nFiles where {} file is less than {} kB:'.format(file, min_size))
#     for li in too_small:
#         print(li)
#
#     print('\nFiles where {} is missing vars:'.format(file))
#     for k, v in incomplete.items():
#         print(k, '\t\t', v)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, Extension, find_packages
import numpy
import os
import sys

setup(
    author="Micah Sandusky",
    author_email='micah.sandusky@ars.usda.gov',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Checking health of programs with consistent directory tree and many outputs",
    license="CC0 1.0",
    # long_description=readme + '\n\n' + history,
    include_package_data=True,
    package_data={'drhawkeye':['./CoreConfig.ini',
				  './recipes.ini']},
    keywords='drhawkeye',
    name='drhawkeye',
    packages=['drhawkeye'],
    test_suite='tests',
    # tests_require=test_requirements,
    url='https://github.com/usdaarsnwrc/drhawkeye',
    version='0.2.4',
    zip_safe=False,
    scripts=[],
)
